/*
 * process.h - Process and breakpoint handling in debugger
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef PROCESS_H
#define PROCESS_H

#include "dwarfx.h"
#include "ptrace.h"

#include "data.h"


struct process {
    struct pt_process *proc;	/* NULL if we've terminated the process */
    struct breakpoint *breakpoints;
    struct breakpoint *current_breakpoint;
        /* current breakpoint; NULL if none */
    struct dwx_obj *objs;
    struct value *value;        /* points back to the value */
    struct process *next;
};

struct breakpoint {
    struct process *process; /* does NOT count as a reference;
                                NULL if process is dead */
    const struct dwx_obj *obj;
    unsigned long addr;
    struct value *value;        /* points back to the value */
    struct breakpoint *next; /* only valid if process != NULL */
};


#include <inttypes.h>

#include "dwarfx.h"
#include "ptrace.h"

#include "data.h"
#include "op.h"
#include "stmt.h"


extern int sbug;
extern int debug;


struct process;
struct breakpoint;
struct object;


/* ----- Context ----------------------------------------------------------- */

void check_process_termination(const struct process *process);
struct process *current_process(void);

/* ----- data.h ------------------------------------------------------------ */

extern struct dwx_type process_type;
extern struct dwx_type breakpoint_type;
extern struct dwx_type location_type;

struct value *process_value(struct process *process);
struct process *__value_process(struct value *value);

struct value *breakpoint_value(struct breakpoint *breakpoint);
struct breakpoint *__value_breakpoint(struct value *value);

int sbug_put(struct value *value);

struct process *get_process(struct process *process);
void put_process(struct process *process);

/* ----- op.h -------------------------------------------------------------- */

struct value *op_run(const struct op *self);
struct value *op_uml(const struct op *self);
struct value *op_breakpoint(const struct op *self);

int destroy_sbug_op(const struct op *stmt);

struct value *op_current_bp(const struct op *self);
struct value *op_current_status(const struct op *self);

int sbug_op_eq(struct value *a,struct value *b,struct value **res);
int sbug_op_ne(struct value *a,struct value *b,struct value **res);

struct value *op_wait(const struct op *self);


/* ----- stmt.h ------------------------------------------------------------ */


void stmt_continue_process(const struct stmt *self);

int destroy_sbug_stmt(const struct stmt *stmt);


#endif /* PROCESS_H */
