/*
 * ptrace.h - Ptrace wrapper
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef PTRACE_H
#define PTRACE_H

#include <inttypes.h>
#include <sys/types.h>

#include "arch.h"


struct pt_break {
    arch_addr_t addr;		/* breakpoint location */
    void *arch_covered;		/* covered opcode */
    int ref;			/* reference count */
    struct pt_break *next;
};

struct pt_process {
    pid_t pid;
    int running;		/* 0 if stopped, exited, or signaled;
				   -1 if starting; 1 if running */
    int terminated;		/* 1 if terminated */
    struct pt_break *bps;	/* breakpoints */
    struct pt_break *stopped;	/* current breakpoint; NULL if none */
    const struct arch_ops *arch_ops;
    void *arch_dsc;
    int status;			/* valid only if terminated */
    struct pt_process *next;
};


/* build descriptor for existing, traced and stopped process */
struct pt_process *pt_new_process(pid_t pid);

struct pt_process *pt_trace(pid_t pid);
struct pt_process *pt_fork(void);
void pt_terminate(struct pt_process *proc); /* kill and/or reap */

void pt_swap_byte(struct pt_process *proc,arch_addr_t addr,uint8_t *byte);

void pt_read(struct pt_process *proc,arch_addr_t addr,void *buf,size_t size);
void pt_write(struct pt_process *proc,arch_addr_t addr,const void *buf,
  size_t size);

void pt_run(struct pt_process *proc);
void pt_break(struct pt_process *proc,arch_addr_t addr);
void pt_unbreak(struct pt_process *proc,arch_addr_t addr);

/*
 * Returns pointer to descriptor of stopped process. If proc->stopped is NULL,
 * the process has terminated, and its status is stored in proc->status. Note
 * that it is still necessary to call pt_terminate.
 *
 * If "proc" is non-NULL, pt_wait will only return when the specified process
 * has stopped.
 */

struct pt_process *pt_wait(const struct pt_process *wait_proc,int block);

#endif /* PTRACE_H */
